/*
 *  Particle.cpp
 *  emptyExample
 *
 *  Created by Andrew Wallace on 9/28/11.
 *  Copyright 2011 Cool Town Inc. All rights reserved.
 *
 */

#include "particle.h"

#include "ofMain.h"

//------------------------------------------------------------
particle::particle(){
	setInitialCondition(0,0,0,0);
	damping = 0.00f;
	lastWall=-1;
}

//------------------------------------------------------------
void particle::resetForce(){
    // we reset the forces every frame
    frc.set(0,0);
}

//------------------------------------------------------------
void particle::addForce(float x, float y){
    // add in a force in X and Y for this frame.
    frc.x = frc.x + x;
    frc.y = frc.y + y;
}

//------------------------------------------------------------
void particle::addDampingForce(){
	
	// the usual way to write this is  vel *= 0.99
	// basically, subtract some part of the velocity 
	// damping is a force operating in the oposite direction of the 
	// velocity vector
	
    frc.x = frc.x - vel.x * damping;
    frc.y = frc.y - vel.y * damping;
}

//------------------------------------------------------------
void particle::setInitialCondition(float px, float py, float vx, float vy){
    pos.set(px,py);
	vel.set(vx,vy);
}

//------------------------------------------------------------
void particle::update(){	
	vel = vel + frc;
	pos = pos + vel;
	//calculate seed
	speed=sqrtf(powf(vel.x,2)+powf(vel.y, 2));
	
	//if it has been a few frames since hitting a wall, set lastWall to -1
	if (--wallTimer <=0)	lastWall=-1;
}

//------------------------------------------------------------
void particle::draw(){
    ofCircle(pos.x, pos.y, 3);
}



void particle::bounce(float bAngle, int _lastWall){
	lastWall=_lastWall;
	float angle=atan2(vel.y,vel.x);
	
	
	//get the angle perpendicular to the wall
	float perpAngle=bAngle+PI/2;
	//and figure out the particle's incoming angle relative to this angle
	float angleDist=perpAngle-angle-PI;
	//add it to the previous angle
	angle-=PI/2-angleDist;
	angle-=PI/2-angleDist;
	//reset the velocity
	vel.x=cos(angle)*speed;
	vel.y=sin(angle)*speed;
	
	//and move it
	//pos.x+=vel.x/2;
	//pos.y+=vel.y/2;
	//pos+=vel;
	wallTimer=3;
	
}