#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup(){
	ofSetVerticalSync(true);
	ofEnableAlphaBlending();
	ofEnableSmoothing();
	ofBackground(255, 255, 255);
	clickDist=5;
	
	//set up the trash bin
	trashX=ofGetWidth()-100;
	trashY=ofGetHeight()-100;
	trashSize=40;
	trashPic.loadImage("trashbin.png");
	
}

//--------------------------------------------------------------
void testApp::update(){
	if (ofGetFrameNum()%100==0){
		particle myParticle;
		myParticle.setInitialCondition(100,0, 1,2);
		particles.push_back(myParticle);
	}		
	
	for (int i = particles.size()-1; i >=0; i--){
		particles[i].resetForce();
		particles[i].addForce(0.0,0.2);	//gravity
		particles[i].addDampingForce();
		particles[i].update();
		
		//check if the particle hit a line
		for (int k=0; k<lines.size(); k++){
			if (particles[i].lastWall!=k && ofDist(particles[i].pos.x,particles[i].pos.y, lines[k].pos1.x, lines[k].pos1.y)<lines[k].length){
				float dist=ofDist(particles[i].pos.x,particles[i].pos.y, lines[k].pos1.x, lines[k].pos1.y);
				if ( ofDist(particles[i].pos.x,particles[i].pos.y, lines[k].pos1.x+cos(lines[k].angle)*dist, lines[k].pos1.y+sin(lines[k].angle)*dist)<particles[i].speed){
					//bounce the particle
					particles[i].bounce(lines[k].angle, k);
					//play the sound
					lines[k].playSound();
				}
				
			}			
		}
		
		//get rid of the old ones
		if (particles[i].pos.y>ofGetHeight()+100)
			particles.erase(particles.begin()+i);
		
	}
	

}

//--------------------------------------------------------------
void testApp::draw(){
	
	//trashbin
	ofSetColor(255, 255, 255,150);
	trashPic.draw(trashX+1, trashY-8);
	
	
	ofSetColor(0,0,0);
	ofFill();
	for (int i = 0; i < particles.size(); i++)
		particles[i].draw();
	
	for (int i = 0; i < lines.size(); i++)
		lines[i].draw();
	
	

}

//--------------------------------------------------------------
void testApp::keyPressed(int key){
		
	lines.clear();
	
	

}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	//if the mouse is over the end of any line, show the handle
	
	for (int i=0; i<lines.size(); i++){
		lines[i].drawHandle1=false;
		lines[i].drawHandle2=false;
		if (ofDist(x, y, lines[i].pos1.x, lines[i].pos1.y)<clickDist) {
			lines[i].drawHandle1=true;
		}
		if (ofDist(x, y, lines[i].pos2.x, lines[i].pos2.y)<clickDist) {
			lines[i].drawHandle2=true;
		}
	}

}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	if (lineEdit!=-1){
		if (pullingPos1)
			lines[lineEdit].setPos1(x,y);
		else
			lines[lineEdit].setPos2(x,y);
		
	}
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	bool editing=false;
	//check to see if the end of a line was selected
	for (int i=0; i<lines.size(); i++){
		if (ofDist(x, y, lines[i].pos1.x, lines[i].pos1.y)<clickDist) {
			lineEdit=i;
			pullingPos1=true;
			editing=true;
		}
		if (ofDist(x, y, lines[i].pos2.x, lines[i].pos2.y)<clickDist) {
			lineEdit=i;
			pullingPos1=false;
			editing=true;
		}
	}
	
	//if there were no lines near the click, add a new line
	if (!editing){
		BounceLine newLine;
		newLine.setup(x, y);
		lines.push_back(newLine);
		
		lineEdit=lines.size()-1;
		pullingPos1=false;
	}
	
		
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	if (lineEdit!=-1){
		//if the end of the line was pulled to the trash, kill it
		if ( (lines[lineEdit].pos1.x > trashX && lines[lineEdit].pos1.x <trashX+trashSize && lines[lineEdit].pos1.y > trashY && lines[lineEdit].pos1.y <trashY+trashSize) ||
			(lines[lineEdit].pos2.x > trashX && lines[lineEdit].pos2.x <trashX+trashSize && lines[lineEdit].pos2.y > trashY && lines[lineEdit].pos2.y <trashY+trashSize) ){
			lines[lineEdit].sound.stop();			//stop the sound
			lines.erase(lines.begin()+lineEdit);	//kill the line
		}
	}
	
	//set it that no lines are being editted
	lineEdit=-1;
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void testApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void testApp::dragEvent(ofDragInfo dragInfo){ 

}